@@ -93,6 +93,7 @@ struct terminal_config {
 	char *pixmap_file;
         char *window_title;                     /* the window title */
 	char *wordclass;			/* select-by-word character class */
+	char *termname;				/* TERM variable setting, store as TERM=xxx */
 } ;
 
 /* Initial command */
@@ -157,6 +158,8 @@ static void parse_an_arg (poptContext state,
 			  const struct poptOption *opt,
 			  const char *arg, void *data);
 
+static void set_hints (GtkWidget *widget);
+
 static void
 about_terminal_cmd (void)
 {
@@ -237,6 +240,9 @@ gnome_term_set_font (ZvtTerm *term, char *font_name)
 			zvt_term_set_fonts  (term, font, font);
 	}
 
+	if (GTK_WIDGET_REALIZED (term))
+		set_hints(term);
+
 	s = gtk_object_get_user_data (GTK_OBJECT (term));
 	if (s)
 		g_free (s);
@@ -440,6 +446,8 @@ load_config (char *class)
 	cfg->pixmap_file = gnome_config_get_string ("pixmap_file");
 	cfg->window_title = NULL;
 
+	cfg->termname = NULL;
+
 	if (strcasecmp (fore_color, back_color) == 0)
 		/* don't let them set identical foreground and background colors */
 		cfg->color_set = 0;
@@ -517,6 +525,7 @@ terminal_config_dup (struct terminal_config *cfg)
 	n->class = g_strdup (cfg->class);
 	n->font = g_strdup  (cfg->font);
 	n->wordclass = g_strdup  (cfg->wordclass);
+	n->termname = g_strdup  (cfg->termname);
 
 	return n;
 }
@@ -527,6 +536,7 @@ terminal_config_free (struct terminal_config *cfg)
 	g_free (cfg->class);
 	g_free (cfg->font);
 	g_free (cfg->wordclass);
+	g_free (cfg->termname);
 	g_free (cfg);
 }
 
@@ -1637,42 +1647,32 @@ button_press (GtkWidget *widget, GdkEventButton *event, ZvtTerm *term)
 	return TRUE;
 }
 
-
 static void
-size_allocate (GtkWidget *widget)
+set_hints (GtkWidget *widget)
 {
         ZvtTerm *term;
-	GnomeApp *app;
-	XSizeHints sizehints;
+	GdkGeometry hints;
+	GtkWidget *app;
 
 	g_assert (widget != NULL);
 	term = ZVT_TERM (widget);
 
-	app = GNOME_APP (gtk_widget_get_toplevel (GTK_WIDGET (term)));
+	app = gtk_widget_get_toplevel(widget);
 	g_assert (app != NULL);
 
 #define PADDING 2
-	sizehints.base_width = 
-	  (GTK_WIDGET (app)->allocation.width) +
-	  (GTK_WIDGET (term)->style->klass->xthickness * 2) -
-	  (GTK_WIDGET (term)->allocation.width) + PADDING;
+	hints.base_width = (GTK_WIDGET (term)->style->klass->xthickness * 2) + PADDING;
+	hints.base_height =  (GTK_WIDGET (term)->style->klass->ythickness * 2);
 
-	sizehints.base_height =
-	  (GTK_WIDGET (app)->allocation.height) +
-	  (GTK_WIDGET (term)->style->klass->ythickness * 2) -
-	  (GTK_WIDGET (term)->allocation.height);
+	hints.width_inc = term->charwidth;
+	hints.height_inc = term->charheight;
+	hints.min_width = hints.base_width + hints.width_inc;
+	hints.min_height = hints.base_height + hints.height_inc;
 
-	sizehints.width_inc = term->charwidth;
-	sizehints.height_inc = term->charheight;
-	sizehints.min_width = sizehints.base_width + sizehints.width_inc;
-	sizehints.min_height = sizehints.base_height + sizehints.height_inc;
-
-	sizehints.flags = (PBaseSize|PMinSize|PResizeInc);
-
-	XSetWMNormalHints (GDK_DISPLAY(),
-			   GDK_WINDOW_XWINDOW (GTK_WIDGET (app)->window),
-			   &sizehints);
-	gdk_flush ();
+	gtk_window_set_geometry_hints(GTK_WINDOW(app),
+				      GTK_WIDGET(term),
+				      &hints,
+				      GDK_HINT_RESIZE_INC|GDK_HINT_MIN_SIZE|GDK_HINT_BASE_SIZE);
 }
 
 
@@ -1742,7 +1742,7 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 
 	/* Setup the environment for the gnome-terminals:
 	 *
-	 * TERM is set to xterm-color (which is what zvt emulates)
+	 * TERM is set to xterm (which is what zvt emulates)
 	 * COLORTERM is set for slang-based applications to auto-detect color
 	 * WINDOWID spot is reserved for the xterm compatible variable.
 	 * COLS is removed
@@ -1756,9 +1756,15 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 		i = p - env;
 		env_copy = (char **) g_malloc (sizeof (char *) * (i + 1 + EXTRA));
 		for (i = 0, p = env; *p; p++){
-			if (strncmp (*p, "TERM=", 5) == 0)
-				env_copy [i++] = "TERM=xterm";
-			else if ((strncmp (*p, "COLUMNS=", 8) == 0)
+			if (strncmp (*p, "TERM=", 5) == 0) {
+				if (cfg->termname)
+					env_copy [i++] = cfg->termname;
+				else
+					/* DO NOT change this, its here for a reason */
+					/* this emulator doesn't emulate xterm-color, or xterm-debian, or
+					   xterm-old, it emulates xterm.  Get it?  Doh! */
+					env_copy [i++] =  "TERM=xterm";
+			} else if ((strncmp (*p, "COLUMNS=", 8) == 0)
 				 || (strncmp (*p, "LINES=", 6) == 0)){
 				/* nothing: do not copy those */
 			} else
@@ -1838,9 +1844,9 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 	gtk_signal_connect (GTK_OBJECT (term), "button_press_event",
 			    GTK_SIGNAL_FUNC (button_press), term);
 
-	gtk_signal_connect_after (GTK_OBJECT (term), "size_allocate",
-				  GTK_SIGNAL_FUNC (size_allocate), term);
-	
+	gtk_signal_connect_after (GTK_OBJECT (term), "realize",
+				  GTK_SIGNAL_FUNC (set_hints), term);
+
 	if (!cfg->menubar_hidden)
 		gnome_app_create_menus_with_data (GNOME_APP (app), gnome_terminal_menu, term);
 	
@@ -1901,9 +1907,6 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 
 	set_color_scheme (term, cfg);
 
-	gdk_window_set_hints (((GtkWidget *)app)->window,
-			      0, 0, 50, 50, 0, 0, GDK_HINT_MIN_SIZE);
-
 	XSync(GDK_DISPLAY(), False);
 
 	errno = 0;
@@ -1985,6 +1988,7 @@ load_session ()
 		
 		/* do this now, since it is only done in a session */
 		cfg->invoke_as_login_shell = gnome_config_get_bool("invoke_as_login_shell");
+		cfg->termname = gnome_config_get_string("termname");
 
 		g_free(class);
 		gnome_config_pop_prefix();
@@ -2068,6 +2072,8 @@ save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style,
 		/* do this now, since it is only done in a session */
 		gnome_config_set_bool("invoke_as_login_shell",
 				      cfg->invoke_as_login_shell);
+		gnome_config_set_string("termname",
+					cfg->termname);
 
 		gnome_config_pop_prefix ();
 		g_free (prefix);
@@ -2116,7 +2122,8 @@ enum {
 	DONOUTMP_KEY = -10,
 	DOWTMP_KEY   = -11,
 	DONOWTMP_KEY = -12,
-        TITLE_KEY    = -13
+        TITLE_KEY    = -13,
+	TERM_KEY     = -14
 };
 
 static struct poptOption cb_options [] = {
@@ -2140,6 +2147,9 @@ static struct poptOption cb_options [] = {
 	{ "command", 'e', POPT_ARG_STRING, NULL, COMMAND_KEY,
 	  N_("Execute this program instead of a shell"), N_("COMMAND")},
 
+	{ "execute", 'x', POPT_ARG_STRING, NULL, COMMAND_KEY,
+	  N_("Execute this program the same was as xterm does"), N_("COMMAND")},
+
 	{ "foreground", '\0', POPT_ARG_STRING, NULL, FORE_KEY,
 	  N_("Foreground color"), N_("COLOR")},
 
@@ -2161,6 +2171,9 @@ static struct poptOption cb_options [] = {
 	{ "title", 't', POPT_ARG_STRING, NULL, TITLE_KEY,
           N_("Set the window title"), N_("TITLE") },
 
+	{ "termname", '\0', POPT_ARG_STRING, NULL, TERM_KEY,
+          N_("Set the TERM variable"), N_("TERMNAME") },
+
 	{ NULL, '\0', 0, NULL, 0}
 };
 
@@ -2232,6 +2245,9 @@ parse_an_arg (poptContext state,
 	case TITLE_KEY:
 	        cfg->window_title = g_strdup(arg);
                 break;
+	case TERM_KEY:
+		cfg->termname = g_strdup_printf("TERM=%s", arg);
+		break;
 	default:
 	}
 }
@@ -2250,6 +2266,7 @@ main_terminal_program (int argc, char *argv [], char **environ)
 	char *program_name;
 	char *class;
 	struct terminal_config *default_config, *cmdline_config;
+	int i, j;
 
 	env = environ;
 	
@@ -2261,6 +2278,28 @@ main_terminal_program (int argc, char *argv [], char **environ)
 	cmdline_config = g_new0 (struct terminal_config, 1);
 	
 	cb_options[0].descrip = (char *)cmdline_config;
+
+	/* pre-scan for -x and --execute options */
+	for (i=1;i<argc;i++) {
+		if (!strcmp(argv[i], "-x") || !(strcmp(argv[i], "--execute"))) {
+			int last=i;
+			i++;
+			if (i==argc) {
+				/* no arg!? let popt whinge about usage */
+				break;
+			}
+			initial_command=malloc((argc-i+1)*sizeof(char *));
+			j = 0;
+			while (i<argc) {
+				initial_command[j] = argv[i];
+				i++; j++;
+			}
+			initial_command[j]=NULL;
+			/* 'fool' popt into thinking we have less args */
+			argc=last;
+			break;
+		}
+	}
 	
 	gnome_init_with_popt_table("Terminal", VERSION, argc, argv,
 				   cb_options, 0, NULL);
@@ -2330,11 +2369,13 @@ main_terminal_program (int argc, char *argv [], char **environ)
 		default_config->color_set = COLORS_CUSTOM;
 	}
 
+
 	/* if the default is different from the commandline, use the commandline */
 	default_config->invoke_as_login_shell =
 		cmdline_login ? cmdline_config->invoke_as_login_shell : 
 		default_config->login_by_default;
-	
+	default_config->termname = g_strdup(cmdline_config->termname);
+
 	terminal_config_free (cmdline_config);
 	
 	if (!load_session ())
